;; apache2.scm -- apache web server

(define usr-sbin-dir "/usr/sbin/")
(define apache (cat usr-sbin-dir "apache2"))

;; normally, we should give that to make-forkexec-constructor with key
;; #:environment-variables, but make-forkexec-constructor uses
;; internally execl instead of execle
(define apache-env (list "APACHE_RUN_USER=www-data"
			 "APACHE_RUN_GROUP=www-data"
			 "APACHE_PID_FILE=/var/run/apache2/apache2.pid"
			 "APACHE_RUN_DIR=/var/run/apache2"
			 "APACHE_LOCK_DIR=/var/lock/apache2"
			 "APACHE_LOG_DIR=/var/log/apache2"
			 ))

;; so we use putenv
(define (set-env)
  (map putenv apache-env))

(define (unset-env)
  (map (lambda (x) (putenv (substring x 0 (string-index x #\=))))
       apache-env))

(define (apache-start)
  (lambda args
    (if (file-exists? apache)
	(begin
	  (boot-format "[....] Starting apache web server...")
	  (unless (file-exists? "/var/run/apache2")
		  (mkdir "/var/run/apache2"))
	  (unless (file-exists? "/var/lock/apache2")
		  (mkdir "/var/lock/apache2"))
	  (chmod "/var/lock/apache2" #o755)
	  (chown "/var/lock/apache2" (passwd:uid (getpwnam "www-data")) -1)

	  ;; TODO: read /etc/apache2/envvars; we cannot use system*
	  ;; because apache needs an environment
	  (set-env)
	  (let ((ret (system (cat apache " -k start"))))
	    (unset-env)
	    (if (zero? ret)
		(begin
		  (boot-format-ok)
		  #t)
		(begin
		  (boot-format-fail)
		  #f)
		)))
	#f)
    ))

(define apache-service
  (make <service>
    #:requires '(rsyslogd)
    #:provides '(apache)
    #:start (apache-start)
    #:stop (lambda (running)
	     (when running
		   (boot-format "[....] Stopping apache web server...")

		   (let ((pid (with-input-from-file "/var/run/apache2/apache2.pid" read)))
		     (set-env)
		     (system (cat apache " -k stop"))
		     (unset-env)

		     ;; the children can take a long time to exit
		     (let ((ret (do ((i 1 (1+ i)))
				    ((or (not (zero? (system (simple-format #f "pgrep -d ' ' -P ~a >/dev/null" pid))))
					 (> i 25))
				     (not (zero? (system* "pgrep" "-d" "' '" "-P" (simple-format #f "~a" pid)))))
				  (sleep 1)
				  (boot-format "*"))))
		       (if ret
			   (boot-format-ok)
			   (boot-format-fail)))))
	     #f)
    ))

(register-services apache-service)
