;; bucardo.scm -- postgresql replication daemon

(define usr-bin-dir "/usr/bin/")
(define bucardo (cat usr-bin-dir "bucardo"))

(define bucardo-piddir "/var/run/bucardo")
(define bucardo-pidfile (cat bucardo-piddir "/bucardo.mcp.pid"))

(define (system-value-read command)
  "Lisp read the string returned by command"
  (let ((res (system-value command)))
    (if (string? res)
	(catch-system-error (with-input-from-string res read))
	res)))

(define bucardo-service
  (make <service>
    #:requires '(postgres)
    #:provides '(bucardo)
    #:start (lambda args
	      (if (file-exists? bucardo)
		  (let ((uid (catch-system-error (passwd:uid (getpwnam "bucardo"))))
			(gid (catch-system-error (group:gid (getgrnam "bucardo")))))
		    (boot-format "[....] Starting bucardo...")
		    
		    (unless (file-exists? bucardo-piddir)
			    (mkdir bucardo-piddir))
		    (if (and uid gid)
			(begin
			  (chown bucardo-piddir uid gid)
			  (let ((pid ((make-forkexec-constructor (list bucardo "start" "shepherd")
								 #:user "bucardo"))))
			    (if pid
				(begin
				  (boot-format-ok)
				  #t)
				(begin
				  (boot-format-fail)
				  #f))))
			(begin
			  (boot-format-fail)
			  #f)))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? bucardo-pidfile))
		   (boot-format "[....] Stopping bucardo...")
		   ((make-forkexec-constructor (list bucardo "stop" "shepherd")
					       #:user "bucardo"))
		   ;; children can take a long time to exit
		   ;; using try-every-second as sleep doesn't work in shepherd 0.10.1 and above
		   ;; we accept success even when not all processes die; probably due to I/O blocking
		   ;; when evetything else is going down during halt/reboot
		   (let ((ret (try-every-second 5 (lambda args
						     (boot-format "*")
						     (let ((i (system-value-read "pgrep -c -u bucardo Bucardo")))
						       (boot-format "~a" i)
						       (<= i 1))))))
		     (if ret
			 (boot-format-ok)
			 (boot-format-fail))))
	     #f)
    ))

(register-services bucardo-service)
