;; cyrus-common.scm -- cyrus mail/nntp service

(define usr-sbin-dir "/usr/sbin/")
(define cyrmaster (cat usr-sbin-dir "cyrmaster"))

(define cyrus-pidfile "/run/cyrus-master.pid")
(define cyrus-conf "/etc/imapd.conf")

(define cyrus-tempfile "/usr/lib/tmpfiles.d/cyrus-imapd.conf")

(define (read-cyrus-tempfile)
  "Parses cyrus-tempfile"
  (let ((s (open-input-file cyrus-tempfile)))
    (do ()
	((eof-object? (peek-char s)))
      (let* ((str (read-line s))
	     (strings (string-tokenize str)))
	(when (and (= 7 (length strings)) (not (char=? #\# (string-ref str 0)))) 
	      (let* ((type (list-ref strings 0))
		     (path (list-ref strings 1))
		     (perms (list-ref strings 2))
		     (octal-perms (with-input-from-string (cat "#o" perms) read))
		     (ucode (list-ref strings 3))
		     (uid (catch-system-error (passwd:uid (getpwnam ucode))))
		     (gcode (list-ref strings 4))
		     (gid (catch-system-error (group:gid (getgrnam gcode))))
		     (npath (list-ref strings 6)))
		(cond
		 ((string=? type "d") (mkdir path) (chmod path octal-perms) (when (and uid gid) (chown path uid gid)))
		 ((string=? type "L") (symlink npath path))
		 (else #f))))))
    #t))

(define (create-socket-dir-from-cyrus-tempfile)
  (if (file-exists? cyrus-tempfile)
      (read-cyrus-tempfile)
      #f))

(define cyrus-service
  (make <service>
    #:requires '(rsyslogd)
    #:provides '(cyrus)
    #:start (lambda args
	      (if (and (file-exists? cyrmaster) (create-socket-dir-from-cyrus-tempfile))
		  ;; TODO: read /etc/default/cyrus-imapd
		  ;; TODO: handle package cyrus-replication
		  (begin
		    (boot-format "[....] Starting cyrus...")
		    (let ((pid ((make-forkexec-constructor (list cyrmaster "-d")))))
		      (if pid
			  (begin
			    (with-output-to-file cyrus-pidfile (lambda () (display pid) (newline)))
			    (boot-format-ok)
			    #t)
			  (begin
			    (boot-format-fail)
			    #f)
			  )))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? cyrus-pidfile))
		   (boot-format "[....] Stopping cyrus...")
		   (let ((pid (with-input-from-file cyrus-pidfile read)))
		     (delete-file cyrus-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok)
		   )
	     #f)
    ))

(register-services cyrus-service)
