;; dbus.scm -- dbus daemon

(define usr-bin-dir "/usr/bin/")
(define dbus (cat usr-bin-dir "dbus-daemon"))

(define dbus-piddir "/var/run/dbus")
(define dbus-pidfile (cat dbus-piddir "/pid"))

(define dbus-service
  (make <service>
    #:requires '(rsyslogd)
    #:provides '(dbus)
    #:start (lambda args
	      (if (file-exists? dbus)
		  (let* ((uid (catch-system-error (passwd:uid (getpwnam "messagebus"))))
			 (gid (catch-system-error (group:gid (getgrnam "messagebus")))))
		    (boot-format "[....] Starting dbus...")
		    
		    ;; TODO: read /etc/default/dbus
		    (when (file-exists? "/usr/bin/dbus-uuidgen")
			  (system* "/usr/bin/dbus-uuidgen" "--ensure"))
		    (unless (file-exists? dbus-piddir)
			    (mkdir dbus-piddir))
		    (if (and uid gid)
			(begin
			  (chown dbus-piddir uid gid)
			  (let ((pid ((make-forkexec-constructor (list dbus "--system") #:pid-file dbus-pidfile))))
			    (if pid
				;; we ensure that we can really talk to dbus before proceeding
				(let* ((cmd "dbus-send --system --type=method_call --dest=org.freedesktop.DBus / org.freedesktop.DBus.Peer.Ping >/dev/null 2>&1")
				       (ret (do ((i 1 (1+ i)))
						((or (zero? (system cmd))
						     (> i 25))
						 (zero? (system cmd)))
					      (sleep 1)
					      (boot-format "*"))))
				  (if ret
				      (begin
					(boot-format-ok)
					#t)
				      (begin
					(boot-format-fail)
					#f)))
				(begin
				  (boot-format-fail)
				  #f)
				)))
			(begin
			  (boot-format-fail)
			  #f)))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? dbus-pidfile))
		   (boot-format "[....] Stopping dbus...")
		   (let ((pid (with-input-from-file dbus-pidfile read)))
		     (delete-file dbus-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    ))

(register-services dbus-service)
