;; exim4-base.scm -- exim4 server

(define usr-sbin-dir "/usr/sbin/")
(define exim4 (cat usr-sbin-dir "exim4"))
(define exim4-pidfile "/run/exim4/exim.pid")

(define (exim4-start)
  (lambda args
    (if (file-exists? exim4)
	(begin
	  (boot-format "[....] Starting exim4 daemon...")
	  
	  (unless (file-exists? "/run/exim4")
		  (mkdir "/run/exim4"))
	  (chmod "/run/exim4" #o750)	    
	  (let* ((uid (catch-system-error (passwd:uid (getpwnam "Debian-exim"))))
		 (gid (catch-system-error (group:gid (getgrnam "Debian-exim")))))
	    (when (and uid gid) (chown "/run/exim4" uid gid)))
	  
	  ;; TODO: read /etc/default/exim4
	  ;; TODO: ! $DAEMON -bV > /dev/null
	  ;; TODO: paniclog et update config

	  ;; exim4 can write its pid file after the delay we have for
	  ;; the #:pid-file keyword argument to
	  ;; make-forkexec-constructor; so add -oX and handle the pid
	  ;; file ourselves
	  (let* ((exim4-cmd (list exim4 "-bd" "-q" "30m" "-oX" "25"))
		 (pid ((make-forkexec-constructor exim4-cmd))))
	    (if pid
		(begin
 		  (with-output-to-file exim4-pidfile (lambda () (display pid) (newline)))
		  (boot-format-ok)
		  #t)
		(begin
		  (boot-format-fail)
		  #f)
		)))
	#f)))

(define exim4-service
  (make <service>
    #:requires '(rsyslogd)
    #:provides '(exim4)
    #:start (exim4-start)
    #:stop (lambda (running)
	     (when (and running (file-exists? exim4-pidfile))
		   (boot-format "[....] Stopping exim4 daemon...")
		   (let ((pid (with-input-from-file exim4-pidfile read)))
		     (delete-file exim4-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    ))

(register-services exim4-service)
