;; fetchmail.scm -- polling mail sources

(define usr-bin-dir "/usr/bin/")
(define fetchmail (cat usr-bin-dir "fetchmail"))

(define fetchmail-conffile "/etc/fetchmailrc")
(define fetchmail-piddir "/var/run/fetchmail")
(define fetchmail-pidfile (cat fetchmail-piddir "/fetchmail.pid"))
(define fetchmail-homedir "/var/lib/fetchmail")

(define (test-perms-owner-conf)
  (and (file-exists? fetchmail-conffile)
       (let ((stats (lstat fetchmail-conffile))
	     (uid (catch-system-error (passwd:uid (getpwnam "fetchmail")))))
	 (and uid
	      (= (stat:uid stats) uid)
	      (= #o600 (stat:perms stats))))))
	
(define (fetchmail-start)
  (lambda args
    (if (and (file-exists? fetchmail) (test-perms-owner-conf))
	(let* ((uid (catch-system-error (passwd:uid (getpwnam "fetchmail"))))
	       (gid (catch-system-error (passwd:gid (getpwnam "fetchmail"))))
	       (gid-nogroup (catch-system-error (group:gid (getgrnam "nogroup"))))
	       (home (catch-system-error (passwd:dir (getpwnam "fetchmail"))))
	       (shell (catch-system-error (passwd:shell (getpwnam "fetchmail")))))
	  (boot-format "[....] Starting fetchmail...")
	  (if (and uid gid gid-nogroup (= gid gid-nogroup)
		   home (string= home fetchmail-homedir)
		   (eq? (stat:type (lstat fetchmail-homedir)) 'directory)
		   (= uid (stat:uid (lstat fetchmail-homedir)))
		   (= gid (stat:gid (lstat fetchmail-homedir)))
		   (= #o700 (stat:perms (lstat fetchmail-homedir)))
		   shell (string= shell "/bin/false"))
	      (begin
		;; TODO: read /etc/default/fetchmail; handle -d 300 and no syslog
		(if (file-exists? fetchmail-pidfile)
		    (begin
		      (boot-format "existing pidfile ~a, fetchmail already running?..."  fetchmail-pidfile)
		      (boot-format-fail)
		      #f)
		    (begin
		      (unless (and (file-exists? fetchmail-piddir)
				   (eq? (stat:type (lstat fetchmail-piddir)) 'directory)
				   (= uid (stat:uid (lstat fetchmail-piddir)))
				   (= gid (stat:gid (lstat fetchmail-piddir)))
				   (= #o700 (stat:perms (lstat fetchmail-piddir))))
			      (if (and (file-exists? fetchmail-piddir)
				       (eq? (stat:type (lstat fetchmail-piddir)) 'directory))
				  (rmdir fetchmail-piddir)
				  (when (file-exists? fetchmail-piddir)
					(delete-file fetchmail-piddir)))
			      (mkdir fetchmail-piddir)
			      (chmod fetchmail-piddir #o700)
			      (chown fetchmail-piddir uid gid))
	    
		      (let* ((fetchmail-cmd (list fetchmail
						  "-f" fetchmail-conffile
						  "--pidfile" fetchmail-pidfile
						  ;; seems necessary despite fetchmail running as user fetchmail
						  "-i" (cat fetchmail-homedir "/.fetchids")
						  "--syslog"))
			     (env (list))
			     (pid (apply (make-forkexec-constructor fetchmail-cmd
								    #:user "fetchmail")
					 args)))
			(if pid
			    (begin
			      (boot-format-ok)
			      #t)
			    (begin
			      (boot-format-fail)
			      #f))))
		    ))
	      (begin
		(boot-format "user fetchmail doesn't exist or not setup properly...")
		(boot-format-fail)
		#f)))
	#f)
    ))

(define fetchmail-service
  (make <service>
    #:requires '(exim4)
    #:provides '(fetchmail)
    #:start (fetchmail-start)
    #:stop (lambda (running)
	     (when (and running (file-exists? fetchmail-pidfile))
		   (boot-format "[....] Stopping fetchmail...")
		   (let ((pid (with-input-from-file fetchmail-pidfile read)))
		     (delete-file fetchmail-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    ))

(register-services fetchmail-service)
