;; fwknopd.scm -- FireWall KNock OPerator server

(define cat string-append)
(define usr-sbin-dir "/usr/sbin/")
(define fwknopd (cat usr-sbin-dir "fwknopd"))
(define fwknopd-piddir "/run/fwknopd")
(define fwknopd-pidfile (cat fwknopd-piddir "/fwknopd.pid"))

(define fwknopd-service
  (make <service>
    #:provides '(fwknopd)
    #:requires '(single-user)
    #:start (lambda args
	      (if (and (file-exists? fwknopd)
		       (file-exists? "/etc/fwknop/access.conf")
		       (file-exists? "/etc/fwknop/fwknopd.conf"))
		  (begin
		    ;; TODO: read /etc/default/fwknop-server
		    (boot-format "[....] Starting fwknopd...")
		    (if (file-exists? fwknopd-pidfile)
			(begin
			  (boot-format "existing pidfile ~a, fwknopd already running?..." fwknopd-pidfile)
			  (boot-format-fail)
			  #f)
			(begin
			  (unless (and (file-exists? fwknopd-piddir)
				       (eq? (stat:type (lstat fwknopd-piddir)) 'directory)
				       (= 0 (stat:uid (lstat fwknopd-piddir)))
				       (= 0 (stat:gid (lstat fwknopd-piddir)))
				       (= #o755 (stat:perms (lstat fwknopd-piddir))))
			    (if (and (file-exists? fwknopd-piddir)
				     (eq? (stat:type (lstat fwknopd-piddir)) 'directory))
				(rmdir fwknopd-piddir)
				(when (file-exists? fwknopd-piddir)
				  (delete-file fwknopd-piddir)))
			    (mkdir fwknopd-piddir)
			    (chmod fwknopd-piddir #o755)
			    (chown fwknopd-piddir 0 0))
			  (let ((pid ((make-forkexec-constructor (list fwknopd)))))
			    (if pid
				(begin
				  (with-output-to-file fwknopd-pidfile (lambda () (display pid) (newline)))
				  (boot-format-ok)
				  #t)
				(begin
				  (boot-format-fail)
				  #f)))
			  )))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? fwknopd-pidfile))
		   (boot-format "[....] Stopping fwknopd...")
		   (let ((pid (with-input-from-file fwknopd-pidfile read)))
		     (delete-file fwknopd-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    )
  )

(register-services fwknopd-service)
