;; lightdm.scm -- X11 session manager

(define usr-sbin-dir "/usr/sbin/")
(define lightdm (cat usr-sbin-dir "lightdm"))

(define lightdm-pidfile "/var/run/lightdm.pid")

(define default-display-manager-file "/etc/X11/default-display-manager")

(define lightdm-service
  (make <service>
    #:requires '(dbus)
    #:provides '(lightdm)
    #:start (lambda args
	      (if (and (file-exists? lightdm)
		       (file-exists? default-display-manager-file)
		       (string= lightdm (symbol->string (with-input-from-file default-display-manager-file read))))
		  (begin
		    (boot-format "[....] Starting lightdm...")
		    (let ((pid ((make-forkexec-constructor (list lightdm)))))
		      (if pid
			  (begin
			    (with-output-to-file lightdm-pidfile (lambda () (display pid) (newline)))
			    (boot-format-ok)
			    #t)
			  (begin
			    (boot-format-fail)
			    #f))))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? lightdm-pidfile))
		   (boot-format "[....] Stopping lightdm...")
		   (let ((pid (with-input-from-file lightdm-pidfile read)))
		     (delete-file lightdm-pidfile)
		     ((make-kill-destructor) pid)
		     (catch-system-error (kill pid SIGTERM))

		     ;; the children can take a long time to exit
		     (let ((ret (do ((i 1 (1+ i)))
				    ((or (not (zero? (system (simple-format #f "pgrep -d ' ' -P ~a >/dev/null" pid))))
					 (> i 15))
				     (not (zero? (system* "pgrep" "-d" "' '" "-P"  (simple-format #f "~a" pid)))))
				  (sleep 1)
				  (boot-format "*"))))
		       (if ret	 
			   (boot-format-ok)
			   (boot-format-fail)))))
	     #f)
    ))

(register-services lightdm-service)
