;; mariadb.scm -- mariadb database

(define usr-sbin-dir "/usr/sbin/")
(define mariadbd (cat usr-sbin-dir "mariadbd"))

(define myadmin-cmd "/usr/bin/mysqladmin")

(define (mariadb-alive? socket)
  (and (file-exists? socket)
       (let ((val (system-value* myadmin-cmd "-s" "ping")))
	 (if (and (string? val) (string= "mysqld is alive" val))
	     #t
	     #f))))

(define (system-value-maria* command . args)
  "Return second line of output when calling (system command)"
  (let* ((port (apply open-pipe* OPEN_READ command args))
	 (str0 (read-line port))
	 (str (read-line port)))
    (close-pipe port)
    str))

(define mysqld "/usr/bin/mysqld_safe")
(define debian-start "/etc/mysql/debian-start")

(define mariadb-service
  (make <service>
    #:requires '(rsyslogd)
    #:provides '(mariadb)
    #:start (lambda args
	      (if (and (file-exists? mariadbd)
		       (file-exists? myadmin-cmd)
		       (file-exists? mysqld)
		       (file-exists? debian-start))
		  (let* ((conf (system-value-maria* mariadbd "--print-defaults"))
			 (assoc-conf (map (lambda (x) (cons (match:substring x 1) (match:substring x 2)))
					  (list-matches "--([^= ]+)=([^ \n]*)[ \n]" conf)))
			 (socket-ret (assoc "socket" assoc-conf))
			 (socket (if socket-ret (cdr socket-ret) "")))
		    (boot-format "[....] Starting mariadb...")
		    (let ((sockdir (system-value* "dirname" socket)))
		      (if (and (file-exists? sockdir)
			       (eq? (stat:type (lstat sockdir)) 'directory))
			  (begin (scan sockdir) (rmdir sockdir))
			  (when (file-exists? sockdir)
			    (delete-file sockdir)))
		      (mkdir sockdir)
		      (chmod sockdir #o755)
		      (let ((uid (catch-system-error (passwd:uid (getpwnam "mysql")))))
			(when uid (chown sockdir uid -1))))
		    
		    ((make-forkexec-constructor (list mysqld) #:pid-file "/run/mysqld/mysqld.pid"))
		    (let ((ret (do ((i 1 (1+ i)))
				   ((or (mariadb-alive? socket)
					(> i 25))
				    (if (> i 25)
					#f
					(mariadb-alive? socket)))
				 (sleep 1)
				 (boot-format "*"))))
		      (boot-format " debian-start...")
		      (system* debian-start)
		      (if ret
			  (begin
			    (boot-format-ok)
			    #t)
			  (begin
			    (boot-format-fail)
			    #f))))
		  #f))
    #:stop (lambda (running)
	     (if (and (file-exists? mariadbd)
		      (file-exists? myadmin-cmd))
		 (let* ((conf (system-value-maria* mariadbd "--print-defaults"))
			(assoc-conf (map (lambda (x) (cons (match:substring x 1) (match:substring x 2)))
					 (list-matches "--([^= ]+)=([^ \n]*)[ \n]" conf)))
			 (socket-ret (assoc "socket" assoc-conf))
			 (socket (if socket-ret (cdr socket-ret) "")))
		   (boot-format "[....] Stopping mariadb...")
		   (let ((ok (system* myadmin-cmd "shutdown")))
		     (when (zero? ok)
		       (begin
			 (when (file-exists? socket) (delete-file socket))
			 (boot-format-ok))
		       (begin
			 (killall mariadbd)
			 (let ((ret (do ((i 1 (1+ i)))
					((or (not (mariadb-alive? socket))
					     (> i 25))
					 (if (> i 25)
                                             #f
					     (not (mariadb-alive? socket))))
				      (sleep 1)
				      (boot-format "*"))))
			   (when (file-exists? socket) (delete-file socket))
			   (if ret
			       (boot-format-ok)
			       (boot-format-fail)))))))
		 #f))
    ))

(register-services mariadb-service)
