;; ndppd.scm -- IPv6 NDP proxy

(define cat string-append)
(define usr-sbin-dir "/usr/sbin/")
(define ndppd (cat usr-sbin-dir "ndppd"))
(define ndppd-piddir "/var/run/ndppd")
(define ndppd-pidfile (cat ndppd-piddir "/ndppd.pid"))

(define ndppd-service
  (make <service>
    #:provides '(ndppd)
    #:requires '(fullnetwork)
    #:start (lambda args
	      (if (and (file-exists? ndppd)
		       (file-exists? "/etc/ndppd.conf"))
		  (begin
		    ;; TODO: read /etc/default/ndppd
		    (boot-format "[....] Starting ndppd...")
		    (if (file-exists? ndppd-pidfile)
			(begin
			  (boot-format "existing pidfile ~a, ndppd already running?..." ndppd-pidfile)
			  (boot-format-fail)
			  #f)
			(begin
			  (unless (and (file-exists? ndppd-piddir)
				       (eq? (stat:type (lstat ndppd-piddir)) 'directory)
				       (= 0 (stat:uid (lstat ndppd-piddir)))
				       (= 0 (stat:gid (lstat ndppd-piddir)))
				       (= #o755 (stat:perms (lstat ndppd-piddir))))
			    (if (and (file-exists? ndppd-piddir)
				     (eq? (stat:type (lstat ndppd-piddir)) 'directory))
				(rmdir ndppd-piddir)
				(when (file-exists? ndppd-piddir)
				  (delete-file ndppd-piddir)))
			    (mkdir ndppd-piddir)
			    (chmod ndppd-piddir #o755)
			    (chown ndppd-piddir 0 0))
			  ((make-forkexec-constructor (list ndppd "-d" "-p" ndppd-pidfile)))
			  (boot-format-ok)
			  #t)))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? ndppd-pidfile))
		   (boot-format "[....] Stopping ndppd...")
		   (let ((pid (with-input-from-file ndppd-pidfile read)))
		     (delete-file ndppd-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    )
  )

(register-services ndppd-service)
