;; nsd.scm -- authoritative domain name server

(define cat string-append)
(define usr-sbin-dir "/usr/sbin/")
(define nsd (cat usr-sbin-dir "nsd"))

(define nsd-tempfile "/usr/lib/tmpfiles.d/nsd.conf")

(define (read-nsd-tempfile)
  "Parses nsd-tempfile"
  (let ((s (open-input-file nsd-tempfile)))
    (do ()
	((eof-object? (peek-char s)))
      (let* ((str (read-line s))
	     (strings (string-tokenize str)))
	(when (and (= 7 (length strings)) (not (char=? #\# (string-ref str 0))))
	      (let* ((type (list-ref strings 0))
		     (path (list-ref strings 1))
		     (perms (list-ref strings 2))
		     (octal-perms (with-input-from-string (cat "#o" perms) read))
		     (ucode (list-ref strings 3))
		     (uid (catch-system-error (passwd:uid (getpwnam ucode))))
		     (gcode (list-ref strings 4))
		     (gid (catch-system-error (group:gid (getgrnam gcode))))
		     (npath (list-ref strings 6)))
		(cond
		 ((string=? type "d") (mkdir path) (chmod path octal-perms) (when (and uid gid) (chown path uid gid)))
		 ((string=? type "L") (symlink npath path))
		 (else #f))))))
    #t))

(define nsd-service
  (make <service>
    #:provides '(nsd)
    #:requires '(fullnetwork)
    #:start (lambda args
	      (if (and (file-exists? nsd) (file-exists? "/etc/nsd/nsd.conf"))
		  (let ((uid (catch-system-error (passwd:uid (getpwnam "nsd"))))
			(nsd-pidfile (system-value* "nsd-checkconf" "-o" "pidfile" "/etc/nsd/nsd.conf")))
		    (boot-format "[....] Starting nsd...")
		    (if (file-exists? nsd-pidfile)
			(begin
			  (boot-format "existing pidfile ~a, nsd already running?..." nsd-pidfile)
			  (boot-format-fail)
			  #f)
			(if uid
			    (begin
			      (read-nsd-tempfile)
			      ((make-forkexec-constructor (list nsd)))
			      (boot-format-ok)
			      #t)
			    (begin
			      (boot-format "user nsd doesn't exist or not setup properly...")
			      (boot-format-fail)
			      #f))))
		  #f))
    #:stop (lambda (running)
	     (when (and running
			(let ((nsd-pidfile (system-value* "nsd-checkconf" "-o" "pidfile" "/etc/nsd/nsd.conf")))
			  (file-exists? nsd-pidfile)))
	       (boot-format "[....] Stopping nsd...")
	       (let ((pid (with-input-from-file nsd-pidfile read)))
		 (delete-file nsd-pidfile)
		 (catch-system-error (kill pid SIGTERM)))
	       (boot-format-ok))
	     #f)
    )
  )

(register-services nsd-service)
