;; sshd.scm -- sshd server

(define usr-sbin-dir "/usr/sbin/")
(define sshd (cat usr-sbin-dir "sshd"))

;; the default; if modified in the config file, must be modified also
;; here
(define ssh-pidfile "/run/sshd.pid")

(define (sshd-start)
  (lambda args
    (if (file-exists? sshd)
	(begin
	  (boot-format "[....] Starting OpenBSD Secure Shell server...")
	  (unless (file-exists? "/run/sshd")
		  (mkdir "/run/sshd")
		  (chmod "/run/sshd" #o755))
	  (if ((make-forkexec-constructor (list sshd)))
	      (begin
		(boot-format-ok)
		#t)
	      (begin
		(boot-format-fail)
		#f)
	      ))
	#f)))

(define sshd-service
  (make <service>
    #:requires '(rsyslogd networking)
    #:provides '(sshd)
    #:start (sshd-start)
    #:stop (lambda (running)
	     (when (and running (file-exists? ssh-pidfile))
		   (boot-format "[....] Stopping OpenBSD Secure Shell server...")
		   (let ((pid (with-input-from-file ssh-pidfile read)))
		     (delete-file ssh-pidfile)
		     ;; ending existing connections
		     ;; pkill is in package procps
		     (system* "/usr/bin/pkill" "-P" (simple-format #f "~a" pid))
		     ;; killing master sshd daemon
		     ;; TODO: handle possible failure here 
		     (catch-system-error (kill pid SIGTERM))
		     )
		   (boot-format-ok))
	     #f)
    ))

(register-services sshd-service)
