;; openvpn.scm -- tunnelling server

(define usr-sbin-dir "/usr/sbin/")
(define openvpn (cat usr-sbin-dir "openvpn"))
(define openvpn-dir "/etc/openvpn/")
(define openvpn-run "/run/openvpn/")

;; TODO: handle client-to-client and /etc/default/openvpn

(define (list-instances)
  (let ((inst (scandir openvpn-dir (lambda (x) (string-match ".*\\.conf$" x)))))
    (and inst (map (lambda (x) (string-drop-right x (string-length ".conf"))) inst))))

(define (list-running-instances)
  (let ((inst (scandir openvpn-run (lambda (x) (string-match ".*\\.pid$" x)))))
    (and inst (map (lambda (x) (string-drop-right x (string-length ".pid"))) inst))))

(define (start-vpn name)
  (let ((openvpn-cmd (list openvpn
			   "--daemon"
			   "--cd" openvpn-dir
			   "--config" (cat openvpn-dir name ".conf")
			   "--writepid" (cat openvpn-run name ".pid"))))
    ((make-forkexec-constructor openvpn-cmd))))

(define openvpn-service
  (make <service>
    #:requires '(networking)
    #:provides '(openvpn)
    #:start (lambda args
	      ;; TODO: handle args as a list of instances
	      (if (file-exists? openvpn)
		  (let ((l (list-instances)))
		    (unless (file-exists? openvpn-run)
			    (mkdir openvpn-run))
		    (and l (map (lambda (x)
				  (boot-format "[....] Starting openvpn ~a..." x)
				  (let ((pid (start-vpn x)))
				    (if pid
					(boot-format-ok)
					(boot-format-fail)
					)))
				l)
			 #t))
		  #f)
	      )
    #:stop (lambda (running)
	     (let ((l (list-running-instances)))
	       (when (and running l)
		     (map (lambda (x)
			    (boot-format "[....] Stopping openvpn ~a..." x)
			    (let ((pid (with-input-from-file (cat openvpn-run x ".pid") read)))
			      (let ((ret (catch-system-error (kill pid SIGTERM)
							     ;; we don't really know how openvpn manages these files
							     (when (file-exists? (cat openvpn-run x ".status"))
								   (delete-file (cat openvpn-run x ".status")))
							     (when (file-exists? (cat openvpn-run x ".pid"))
								   (delete-file (cat openvpn-run x ".pid"))))))
				(if ret
				    (boot-format-ok)
				    (boot-format-fail)))))
			  l))
	       #f))
    )
  )
 
(register-services openvpn-service)
