;; php-fpm.scm -- fastcgi pour php

(define cat string-append)

(define usr-sbin-dir "/usr/sbin/")
(define php-fpm (cat usr-sbin-dir "php-fpm7.4"))

(define conf-file "/etc/php/7.4/fpm/php-fpm.conf")
(define php-fpm-piddir "/run/php")
(define php-fpm-pidfile (cat php-fpm-piddir "/php7.4-fpm.pid")) ;; should be read from configuration file

(define (create-dir-php dir uid gid)
  "uid and gid can be integers or strings"
  (unless (and (file-exists? dir)
	       (eq? (stat:type (lstat dir)) 'directory))
    (if (file-exists? dir)
	(delete-file dir)
	(mkdir dir)))
  (chmod dir #o750)
  (chown dir
	 (if (string? uid)
	     (let ((r (catch-system-error (passwd:uid (getpwnam uid)))))
	       (if r r 0))
	     uid)
	 (if (string? gid)
	     (let ((r (catch-system-error (group:gid (getgrnam gid)))))
	       (if r r 0))
	     gid)
	 ))

(define (system-value-read-php command)
  "Lisp read the string returned by command"
  (let ((res (system-value command)))
    (if (string? res)
	(catch-system-error (with-input-from-string res read))
	res)))

(define php-fpm-service
  (make <service>
    #:provides '(php-fpm)
    #:requires '(apache)
    #:start (lambda args
	      (if (file-exists? php-fpm)
		  (begin
		    (boot-format "[....] Starting php-fpm...")
		    (create-dir-php php-fpm-piddir "www-data" "www-data")
		    (if ((make-forkexec-constructor (list php-fpm "--fpm-config" conf-file)))
			(begin
			  (boot-format-ok)
			  #t)
			(begin
			  (boot-format-fail)
			  #f)
			))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? php-fpm-pidfile))
	       (boot-format "[....] Stopping php-fpm...")
	       (let ((pid (with-input-from-file php-fpm-pidfile read)))
		 (catch-system-error (kill pid SIGQUIT))
		 ;; children can take a long time to exit
		 (let ((ret (do ((i 1 (1+ i)))
				((or (<= (system-value-read-php "pgrep -c -d ' ' php-fpm7.4") 1)
				     (> i 5))
				 (if (> i 5)
				     #f
				     #t))
			      (sleep 1)
			      (boot-format "*"))))
		   (if ret
		       (boot-format-ok)
		       (begin
			 (catch-system-error (kill pid SIGTERM))
			 (let ((ret (do ((i 1 (1+ i)))
					((or (<= (system-value-read-php "pgrep -c -d ' ' php-fpm7.4") 0)
					     (> i 5))
					 (if (> i 5)
					     (catch-system-error (kill pid SIGKILL))
					     #t))
				      (sleep 1)
				      (boot-format "*"))))
			   (if ret
			       (begin
				 (when (file-exists? php-fpm-pidfile)
				   (delete-file php-fpm-pidfile))
				 (boot-format-ok))
			       (boot-format-fail))))))))
	     #f)
    ))

(register-services php-fpm-service)
