;; postgresql-common.scm -- postgresql database

(define postgresql-bindir "/usr/lib/postgresql/")
(define postgresql-piddir "/var/run/postgresql")

(define (create-socket-directory)
  (let* ((uid (catch #t (lambda () (passwd:uid (getpwnam "postgres"))) (lambda (key . args) #f)))
	 (gid (catch #t (lambda () (group:gid (getgrnam "postgres"))) (lambda (key . args) #f))))
    (if (and uid gid)
	(begin
	  (unless (and (file-exists? postgresql-piddir)
		       (eq? (stat:type (lstat postgresql-piddir)) 'directory))
		  (when (file-exists? postgresql-piddir)
		        (delete-file postgresql-piddir))
		  (mkdir postgresql-piddir))
	  (chmod postgresql-piddir #o2775)
	  (chown postgresql-piddir uid gid)
	  ;; TODO: SELinux restorecon
	  #t)
	#f)))

(define (get-versions)
  (let ((versions '()))
    (for-each (lambda (x)
		(when (file-exists? (cat postgresql-bindir x "/bin/pg_ctl"))
		      (set! versions (cons x versions))))
	      (or (scandir postgresql-bindir (lambda (x) (string-match "[0-9]+.*" x))) '()))
    versions))

(define postgresql-service
  (make <service>
    #:requires '(rsyslogd)
    #:provides '(postgres)
    #:start (lambda args
	      (let ((l (get-versions)))
		(if (and l (create-socket-directory))
		    (begin
		      (for-each (lambda (x)
				  (when (and (file-exists? (cat postgresql-bindir x "/bin/postgres"))
					     (file-exists? (cat "/etc/postgresql/" x)))
					(let ((clusters (scandir (cat "/etc/postgresql/" x)
								 (lambda (x) (string-match "[^.]+.*" x)))))
					  (for-each (lambda (clus)
						      (boot-format "[....] Starting postgresql ~a cluster ~a ..."
								   x clus)
						      (if ((make-system-constructor "pg_ctlcluster " x " " clus " start -- -w")) 
							  (boot-format-ok)
							  (boot-format-fail))
						      )
						    (or clusters '())))))
				l)
		      #t)
		    #f)))
    #:stop (lambda (running)
	      (let ((l (get-versions)))
		(when l
		      (for-each (lambda (x)
				  (when (and (file-exists? (cat postgresql-bindir x "/bin/postgres"))
					     (file-exists? (cat "/etc/postgresql/" x)))
					(let ((clusters (scandir (cat "/etc/postgresql/" x)
								 (lambda (x) (string-match "[^.]+.*" x)))))
					  (for-each (lambda (clus)
						      (boot-format "[....] Stopping postgresql ~a cluster ~a ..."
								   x clus)
						      (if ((make-system-constructor "pg_ctlcluster " "--force " x " " clus " stop"))
							  (boot-format-ok)
							  (boot-format-fail))
						      )
						    (or clusters '())))))
				l))
		#f))
    ))

(register-services postgresql-service)
