;; radvd.scm -- IPv6 router advertisement

(define cat string-append)
(define usr-sbin-dir "/usr/sbin/")
(define radvd (cat usr-sbin-dir "radvd"))
(define radvd-piddir "/var/run/radvd")
(define radvd-pidfile (cat radvd-piddir "/radvd.pid"))

(define ipv6-forwarding "/proc/sys/net/ipv6/conf/all/forwarding")

(define radvd-service
  (make <service>
    #:provides '(radvd)
    #:requires '(fullnetwork)
    #:start (lambda args
	      (if (and (file-exists? radvd) (file-exists? "/etc/radvd.conf"))
		  (let ((uid (catch-system-error (passwd:uid (getpwnam "radvd")))))
		    (boot-format "[....] Starting radvd...")
		    (if (file-exists? radvd-pidfile)
			(begin
			  (boot-format "existing pidfile ~a, radvd already running?..." radvd-pidfile)
			  (boot-format-fail)
			  #f)
			(if uid
			    (if (and (file-exists? ipv6-forwarding) (zero? (with-input-from-file ipv6-forwarding read)))
				(begin
				  (boot-format "not starting because no IPv6 forwarding...")
				  (boot-format-fail)
				  #f)
				(when (file-exists? ipv6-forwarding)
				  (unless (and (file-exists? radvd-piddir)
					       (eq? (stat:type (lstat radvd-piddir)) 'directory)
					       (= uid (stat:uid (lstat radvd-piddir)))
					       (= 0 (stat:gid (lstat radvd-piddir)))
					       (= #o4755 (stat:perms (lstat radvd-piddir))))
				    (if (and (file-exists? radvd-piddir)
					     (eq? (stat:type (lstat radvd-piddir)) 'directory))
					(rmdir radvd-piddir)
					(when (file-exists? radvd-piddir)
					  (delete-file radvd-piddir)))
				    (mkdir radvd-piddir)
				    (chmod radvd-piddir #o4755)
				    (chown radvd-piddir uid 0))
				  ((make-forkexec-constructor (list radvd
								    "-u" "radvd"
								    "-p" radvd-pidfile
								    "-m" "syslog")))
				  (boot-format-ok)
				  #t))
			    (begin
			      (boot-format "user radvd doesn't exist or not setup properly...")
			      (boot-format-fail)
			      #f))))
		  #f))
    #:stop (lambda (running)
	     (when (and running (file-exists? radvd-pidfile))
		   (boot-format "[....] Stopping radvd...")
		   (let ((pid (with-input-from-file radvd-pidfile read)))
		     (delete-file radvd-pidfile)
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    )
  )

(register-services radvd-service)
