;; rsyslog.scm -- syslog daemon

(define usr-sbin-dir "/usr/sbin/")
(define rsyslogd (cat usr-sbin-dir "rsyslogd"))
(define rsyslogd-pidfile "/var/run/rsyslogd.pid")

(define (rsyslogd-start)
  (lambda args
    (if (file-exists? rsyslogd)
	(begin
	  (boot-format "[....] Starting rsyslog...")
	  (unless (file-exists? "/dev/xconsole")
	    (mknod "/dev/xconsole" 'fifo #o640 0)
	    (let ((gid (catch-system-error (group:gid (getgrnam "adm")))))
	      (when gid (chown "/dev/xconsole" -1 gid))))
	  (if ((make-forkexec-constructor (list rsyslogd) #:pid-file rsyslogd-pidfile))
	      (begin
		(when (file-exists? "/run/sendsigs.omit.d")
		  (symlink rsyslogd-pidfile "/run/sendsigs.omit.d/rsyslog"))
		(boot-format-ok)
		(let ((ret (do ((i 1 (1+ i)))
				    ((or (file-exists? "/dev/log")
					 (> i 5))
				     (file-exists? "/dev/log"))
			     (sleep 1)
			     (boot-format "*"))))
		  ;; that call to boot-format forces the switch to
		  ;; logging through /dev/log; it is needed to allow
		  ;; shepherd message "Service rsyslogd has been
		  ;; started" to appear at the right place
		  (boot-format "[....] /dev/log created...")
		  (if ret
		      (boot-format-ok)
		      (boot-format-fail)))
		(with-input-from-file "/run/sendsigs.omit.d/rsyslog" read)
		)
	      (begin
		(boot-format-fail)
		#f)
	      ))
	#f
	)))

(define rsyslogd-service
  (make <service>
    #:requires '(single-user)
    #:provides '(rsyslogd)
    #:start (rsyslogd-start)
    #:stop (lambda (running)
	     (when running
	       ;; so boot-format won't write to it and keep rsyslog alive
	       (delete-file "/dev/log")
	       (when (file-exists? "dev/log")
		 (boot-format "Failed to delete /dev/log\n"))
	       (boot-format "[....] Stopping rsyslog...")
	       (catch 'system-error
		      (lambda ()
			(kill running SIGTERM)
			(delete-file "/run/sendsigs.omit.d/rsyslog")
			(boot-format-ok))
		      (lambda (key . args) (boot-format-fail))))
	     #f)
    )
  )

(register-services rsyslogd-service)
