;; unbound.scm -- unbound recursive DNS server

(define usr-sbin-dir "/usr/sbin/")
(define unbound (cat usr-sbin-dir "unbound"))

(define unbound-service
  (make <service>
    #:provides '(unbound)
    #:requires '(fullnetwork)
    #:start (lambda args
	      (if (file-exists? unbound)
		  (begin
		    (boot-format "[....] Starting unbound...")
		    (let ((ret (status:exit-val (system-pipe "/usr/sbin/unbound-anchor -a /var/lib/unbound/root.key -v"
							     (lambda (x) (boot-format "~a..." x))))))
		      (if (zero? ret)
			  (boot-format "root anchor updated...")
			  (boot-format "root anchor unchanged...")))
		    (let* ((uid (catch-system-error (passwd:uid (getpwnam "unbound"))))
			   (gid (catch-system-error (group:gid (getgrnam "unbound")))))
		      (when (and uid gid) (chown "/var/lib/unbound/root.key" uid gid)))
		    (let ((pid (apply (make-forkexec-constructor (list unbound)) args)))
		      (with-output-to-file "/var/run/unbound.pid" (lambda () (display pid) (newline)))
		      )
		    (boot-format-ok)
		    #t)
		  #f)
	      )
    #:stop (lambda (running)
	     (when (and running (file-exists? "/var/run/unbound.pid"))
		   (boot-format "[....] Stopping unbound...")
		   (let ((pid (with-input-from-file "/var/run/unbound.pid" read)))
		     (delete-file "/var/run/unbound.pid")
		     (catch-system-error (kill pid SIGTERM)))
		   (boot-format-ok))
	     #f)
    )
  )

(register-services unbound-service)


