;; zoneminder.scm -- video surveillance

(define usr-bin-dir "/usr/bin/")
(define command (cat usr-bin-dir "zmpkg.pl"))

(define rundir "/var/run/zm")
(define tmpdir "/tmp/zm")

(define (create-dir dir uid gid)
  "uid and gid can be integers or strings"
  (scan dir) ;; needed if we want rmdir to succeed
  (if (and (file-exists? dir)
	   (eq? (stat:type (lstat dir)) 'directory))
	(rmdir dir)
	(when (file-exists? dir)
	  (delete-file dir)))
    (mkdir dir)
    (chmod dir #o755)
    (chown dir
	   (if (string? uid)
	       (let ((r (catch-system-error (passwd:uid (getpwnam uid)))))
		 (if r r 0))
	       uid)
	   (if (string? gid)
	       (let ((r (catch-system-error (group:gid (getgrnam gid)))))
		 (if r r 0))
	       gid)
	   ))

(define zoneminder-service
  (make <service>
    #:requires '(mariadb)
    #:provides '(zoneminder)
    #:start (lambda args
              (if (file-exists? command)
		  (begin	
		    (boot-format "[....] Starting zoneminder...")
		    (create-dir rundir "www-data" "www-data")
		    (create-dir tmpdir "www-data" "www-data")
		    (let ((ret (system* command "start")))
		      (if (zero? ret)
			  (begin
			    (close (open-output-file "/var/lock/zm"))
			    (boot-format-ok)
			    #t)
			  (begin
			    (boot-format-fail)
			    #f))))
		  #f))
    #:stop (lambda (running)
	     (if (file-exists? command)
		 (begin	
		   (boot-format "[....] Stopping zoneminder...")
		   (let ((val (system-value* command "status")))
		     (if (and (string? val) (string= "running" val))
			 (begin
			   (let ((ret (system* command "stop")))
			     (if (zero? ret)
				 (boot-format-ok)
				 (boot-format-fail))))
			 (begin
			   (boot-format "already stopped...")
			   (boot-format-ok))))
		   (scan rundir)
		   (when (file-exists? "/var/lock/zm")
		     (delete-file "/var/lock/zm"))
		   )
		 #f))
    ))

(register-services zoneminder-service)
