;; fuser.scm -- kill utilities

(define (killall5 sig . args)
  "Kill all processes with signal sig, except those in args (a list of pid)"
  (let ((procs (scandir "/proc" (lambda (x)
				  (string-match "[0-9]+" x)))))
    ;; procs doesn't contain user threads, but strangely kernel threads
    (for-each (lambda (s) (let ((pid (with-input-from-string s read)))
			    (unless (or (= 1 pid) (member pid args) (not (catch-system-error (readlink (cat "/proc/" s "/exe")))))
			      (boot-format "killing ~a..." pid) 
			      (catch-system-error (kill pid sig)))))
	      procs)))

(define (killall path)
  "Kill all processes with executable path"
  (let ((procs (scandir "/proc" (lambda (x)
				  (string-match "[0-9]+" x)))))
    ;; procs doesn't contain user threads, but strangely kernel threads
    (for-each (lambda (s) (let* ((pid (with-input-from-string s read))
				 (exe (catch-system-error (readlink (cat "/proc/" s "/exe")))))
			    (when (and exe (string=? path exe))
				  (boot-format "killing ~a..." path) 
				  (catch-system-error (kill pid SIGTERM)))))
	      procs)))

(define (fuser)
  "Returns list of pids of processes having a file open writable on /"
  (let ((mntid (rootfs-mntid))
	(result '())
	(procs (scandir "/proc" (lambda (x) (string-match "[0-9]+" x)))))
    (for-each (lambda (s)
		;; s may have disappeared since previous scandir
		(catch-system-error
		 (let ((fds (scandir (cat "/proc/" s "/fdinfo") (lambda (x) (string-match "[0-9]+" x)))))
		   (for-each (lambda (f)
			       (let ((flags #f)
				     (ok #t))
				 ;; fds may have disappeared since previous scandir
				 (catch-system-error
				  (with-input-from-file (cat "/proc/" s "/fdinfo/" f)
				    (lambda ()
				      (do ((str (read-line) (read-line)))
					  ((eof-object? str))
					(let* ((args (string-split str #\:))
					       (key (car args))
					       (arg (cadr args)))
					  ;; not in / filesystem
					  (when (and ok (string= "mnt_id" key))
					    (when (= mntid (string->number (string-trim arg)))
					      (unless (member (string->number s) result)
						(set! result (cons (string->number s) result)))
					      (boot-format "File still open for write: pid ~a fd ~a: ~a (~a)\n"
							   s f flags
							   ;; f may have disappeared since previous scandir
							   (catch-system-error (readlink (cat "/proc/" s "/fd/" f))))
					      ))
					  ;; not writable
					  (when (string= "flags" key)
					    (set! flags arg)
					    (set! ok (not (zero? (logand 3 (string->number (string-trim arg) 8))))))
					  )))))))
			    fds))))
	      procs)
    result))
			
			  
		      
