;; utmp.scm -- fill /run/utmp and /var/log/wtmp

(add-to-load-path (dirname (current-filename)))

(import (prefix (system foreign) ffi:))
(use-modules (rnrs bytevectors))
(use-modules (ffi utmp) (bytestructures guile) (bytestructures guile ffi))
(use-modules (ice-9 receive))

;; not using generated function templates to allow error messages with
;; second argument err to receive
;; (use-modules (system ffi-help-rt))

(define RUN_LVL 1)
(define BOOT_TIME 2)

(define utmpname
  (let ((this (dynamic-link)))
    (ffi:pointer->procedure ffi:void
                        (dynamic-func "utmpname" this)
                        (list '*)
			#:return-errno? #t)))

(define setutent
  (let ((this (dynamic-link)))
    (ffi:pointer->procedure ffi:void
                        (dynamic-func "setutent" this)
                        (list)
			#:return-errno? #t)))

(define endutent
  (let ((this (dynamic-link)))
    (ffi:pointer->procedure ffi:void
                        (dynamic-func "endutent" this)
                        (list)
			#:return-errno? #t)))

(define pututline
  (let ((this (dynamic-link)))
    (ffi:pointer->procedure '*
			    (dynamic-func "pututline" this)
			    (list '*)
			    #:return-errno? #t)))

(define updwtmp
  (let ((this (dynamic-link)))
    (ffi:pointer->procedure ffi:void
                        (dynamic-func "updwtmp" this)
                        (list '* '*)
			#:return-errno? #t)))

(define (runlevel-set! file user id pid type line)
  (let ((entry (bytestructure struct-utmp-desc)))
    (bytestructure-set! entry 'ut_type type)
    (bytestructure-set! entry 'ut_pid pid)
    (bytestructure-set! entry 'ut_user (list->vector (map (lambda (x)
							    (char->integer x))
							  (string->list user))))
    (bytestructure-set! entry 'ut_host (list->vector (map (lambda (x)
							    (char->integer x))
							  (string->list (utsname:release (uname))))))
    (bytestructure-set! entry 'ut_line (list->vector (map (lambda (x)
							    (char->integer x))
							  (string->list line))))
    (bytestructure-set! entry 'ut_id (list->vector (map (lambda (x)
							  (char->integer x))
							(string->list id))))
    (let ((tim (gettimeofday)))
      (bytestructure-set! entry 'ut_tv 'tv_sec (car tim)) 
      (bytestructure-set! entry 'ut_tv 'tv_usec (car tim)))

    ;; could be (utmpname file) with (use-modules (system ffi-help-rt))
    (utmpname (ffi:string->pointer file))  
    (setutent)
    (receive (ret err)
	;; could be (pututline (pointer-to entry)) with (use-modules (system ffi-help-rt))
	(pututline (ffi:bytevector->pointer (bytestructure-bytevector entry)))
      (when (ffi:null-pointer? ret)
	(display (strerror err)) (display "; failed trying to write to ") (display file) (newline))
      (endutent)
      (= err 0))
    ))

(define (wtmp-runlevel-set! file user id pid type line)
  (let ((entry (bytestructure struct-utmp-desc)))
    (bytestructure-set! entry 'ut_type type)
    (bytestructure-set! entry 'ut_pid pid)
    (bytestructure-set! entry 'ut_user (list->vector (map (lambda (x)
							    (char->integer x))
							  (string->list user))))
    (bytestructure-set! entry 'ut_host (list->vector (map (lambda (x)
							    (char->integer x))
							  (string->list (utsname:release (uname))))))
    (bytestructure-set! entry 'ut_line (list->vector (map (lambda (x)
							    (char->integer x))
							  (string->list line))))
    (bytestructure-set! entry 'ut_id (list->vector (map (lambda (x)
							  (char->integer x))
							(string->list id))))
    (let ((tim (gettimeofday)))
      (bytestructure-set! entry 'ut_tv 'tv_sec (car tim)) 
      (bytestructure-set! entry 'ut_tv 'tv_usec (car tim)))

    ;; could be (updwtmp file (pointer-to entry)) with (use-modules (system ffi-help-rt))
    (updwtmp (ffi:string->pointer file)
	     (ffi:bytevector->pointer (bytestructure-bytevector entry)))
    ))

(define runlevel (char->integer #\4))

